@@ -68,22 +68,22 @@ void PartMan::populate(QTreeWidgetItem *drvstart)
     QTreeWidgetItem *curdrv = nullptr;
     for (const BlockDeviceInfo &bdinfo : listBlkDevs) {
         if(bdinfo.isBoot && !brave) continue;
-        QTreeWidgetItem *curdev;
+        QTreeWidgetItem *curdev = nullptr;
         if (bdinfo.isDrive) {
             if(!drvstart) curdrv = new QTreeWidgetItem(gui.treePartitions);
             else if(bdinfo.name == drvstart->text(Device)) curdrv = drvstart;
             else if(!curdrv) continue; // Skip until the drive is drvstart.
             else break; // Exit the loop early if the drive isn't drvstart.
+            drvitMarkLayout(curdrv, true);
             curdev = curdrv;
-            curdrv->setData(Device, Qt::UserRole, QVariant(false)); // drive starts off "unused"
             // Model
             curdev->setText(Label, bdinfo.model);
         } else {
             if(!curdrv) continue;
             curdev = new QTreeWidgetItem(curdrv);
             setupItem(curdev, &bdinfo);
-            curdrv->setData(Device, Qt::UserRole, QVariant(true)); // drive is now "used"
         }
+        assert(curdev!=nullptr);
         // Device name and size
         curdev->setText(Device, bdinfo.name);
         curdev->setText(Size, QLocale::system().formattedDataSize(bdinfo.size, 1, QLocale::DataSizeTraditionalFormat));
@@ -93,9 +93,15 @@ void PartMan::populate(QTreeWidgetItem *drvstart)
     for (int ixi = gui.treePartitions->columnCount() - 1; ixi >= 0; --ixi) {
         if(ixi != Label) gui.treePartitions->resizeColumnToContents(ixi);
     }
+    comboTypeTextChange(QString()); // For the badblocks checkbox.
     gui.treePartitions->blockSignals(false);
     treeSelChange();
 }
+void PartMan::clearLayout(QTreeWidgetItem *drvit)
+{
+    while(drvit->childCount()) drvit->removeChild(drvit->child(0));
+    drvitMarkLayout(drvit, false);
+}
 
 bool PartMan::manageConfig(MSettings &config, bool save)
 {
@@ -105,14 +111,13 @@ bool PartMan::manageConfig(MSettings &config, bool save)
         const QString &drvDevice = drvit->text(Device);
         // Check if the drive is to be cleared and formatted.
         int partCount = drvit->childCount();
-        bool drvPreserve = drvit->data(Device, Qt::UserRole).toBool();
+        bool drvPreserve = twitIsOldLayout(drvit, false);
         const QString &configNewLayout = "Storage/NewLayout."+drvDevice;
         if(save) {
             if(!drvPreserve) config.setValue(configNewLayout, partCount);
         } else if(config.contains(configNewLayout)) {
             drvPreserve = false;
-            while(drvit->childCount()) drvit->removeChild(drvit->child(0));
-            drvit->setData(Device, Qt::UserRole, QVariant(false));
+            clearLayout(drvit);
             partCount = config.value(configNewLayout).toInt();
         }
         // Partition configuration.
@@ -140,7 +145,7 @@ bool PartMan::manageConfig(MSettings &config, bool save)
                 if(totalMB > maxMB) config.markBadWidget(spinSize);
             }
             config.manageComboBox("UseFor", twitComboBox(twit, UseFor), false);
-            config.manageComboBox("Format", twitComboBox(twit, Type), true);
+            config.manageComboBox("Format", twitComboBox(twit, Format), true);
             if(!save && config.contains("Encrypt")) {
                 const bool crypto = config.value("Encrypt").toBool();
                 twit->setCheckState(Encrypt, crypto ? Qt::Checked : Qt::Unchecked);
@@ -211,10 +216,10 @@ void PartMan::setupItem(QTreeWidgetItem *twit, const BlockDeviceInfo *bdinfo,
     // Type
     QComboBox *comboType = new QComboBox(gui.treePartitions);
     comboType->setAutoFillBackground(true);
-    gui.treePartitions->setItemWidget(twit, Type, comboType);
+    gui.treePartitions->setItemWidget(twit, Format, comboType);
     comboType->setEnabled(false);
     if(bdinfo) {
-        twit->setText(Type, bdinfo->fs);
+        twit->setText(Format, bdinfo->fs);
         comboType->addItem(bdinfo->fs);
     }
     connect(comboType, &QComboBox::currentTextChanged, this, &PartMan::comboTypeTextChange);
@@ -234,6 +239,9 @@ void PartMan::labelParts(QTreeWidgetItem *drive)
     for(int ixi = drive->childCount() - 1; ixi >= 0; --ixi) {
         drive->child(ixi)->setText(Device, BlockDeviceInfo::join(drv, ixi+1));
     }
+    for (int ixi = gui.treePartitions->columnCount() - 1; ixi >= 0; --ixi) {
+        if(ixi != Label) gui.treePartitions->resizeColumnToContents(ixi);
+    }
 }
 
 QString PartMan::translateUse(const QString &alias)
@@ -242,8 +250,17 @@ QString PartMan::translateUse(const QString &alias)
     else if(alias == "boot") return QStringLiteral("/boot");
     else if(alias == "home") return QStringLiteral("/home");
     else if(!alias.compare("ESP", Qt::CaseInsensitive)) return QStringLiteral("ESP");
+    else if(!alias.compare("SWAP", Qt::CaseInsensitive)) return QStringLiteral("SWAP");
     else return alias;
 }
+QString PartMan::describeUse(const QString &use)
+{
+    if(use == "/") return "/ (root)";
+    else if(use == "ESP") return tr("EFI System Partition");
+    else if(use == "SWAP") return tr("swap space");
+    else if(use.startsWith("SWAP")) return tr("swap space #%1").arg(use.mid(4));
+    return use;
+}
 
 void PartMan::setEncryptChecks(const QString &use,
     enum Qt::CheckState state, QTreeWidgetItem *exclude)
@@ -281,7 +298,7 @@ void PartMan::comboUseTextChange(const QString &text)
     if(!combo) return;
     QTreeWidgetItem *item = static_cast<QTreeWidgetItem *>(combo->property("row").value<void *>());
     if(!item) return;
-    QLineEdit *editLabel = static_cast<QLineEdit *>(gui.treePartitions->itemWidget(item, Label));
+    QLineEdit *editLabel = twitLineEdit(item, Label);
 
     const QString &usetext = translateUse(text);
     int useClass = -1;
@@ -289,22 +306,29 @@ void PartMan::comboUseTextChange(const QString &text)
     else if(usetext == "ESP") useClass = 1;
     else if(usetext == "bios_grub") useClass = 2;
     else if(usetext == "/boot") useClass = 3;
-    else if(usetext == "swap") useClass = 4;
+    else if(usetext == "SWAP") useClass = 4;
     else useClass = 5;
     int oldUseClass = combo->property("class").toInt();
+    bool selPreserve = false;
     if(useClass != oldUseClass) {
-        QComboBox *comboType = twitComboBox(item, Type);
+        QComboBox *comboType = twitComboBox(item, Format);
         comboType->clear();
-        const QString &curtype = item->text(Type);
+        const QString &curtype = item->text(Format);
         switch(useClass) {
         case 0:
             editLabel->setText(item->text(Label));
             comboType->addItem(curtype);
             break;
-        case 1: comboType->addItem("FAT32", "FAT32"); break;
+        case 1:
+            comboType->addItem("FAT32", "FAT32");
+            selPreserve = true;
+            break;
         case 2: comboType->addItem("GRUB", "GRUB"); break;
         case 3: comboType->addItem("ext4", "ext4"); break;
-        case 4: comboType->addItem("SWAP", "SWAP"); break;
+        case 4:
+            comboType->addItem("SWAP", "SWAP");
+            selPreserve = true;
+            break;
         default:
             comboType->addItem("ext4", "ext4");
             comboType->addItem("ext3", "ext3");
@@ -328,6 +352,7 @@ void PartMan::comboUseTextChange(const QString &text)
             // Add an item at the start to allow preserving the existing format.
             comboType->insertItem(0, tr("Preserve (%1)").arg(curtype), "PRESERVE");
             comboType->insertSeparator(1);
+            if(selPreserve) comboType->setCurrentIndex(0);
         }
         comboType->setEnabled(comboType->count()>1);
         // Label and options
@@ -335,7 +360,7 @@ void PartMan::comboUseTextChange(const QString &text)
         gui.treePartitions->itemWidget(item, Options)->setEnabled(useClass!=0);
         combo->setProperty("class", QVariant(useClass));
     }
-    editLabel->setText(defaultLabels.value(usetext));
+    if(useClass!=0) editLabel->setText(defaultLabels.value(usetext));
     gui.treePartitions->blockSignals(false);
     treeItemChange(item, UseFor);
 }
@@ -345,14 +370,14 @@ void PartMan::comboTypeTextChange(const QString &)
     bool canCheckBlocks = false;
     for(; *it; ++it) {
         if(twitUseFor(*it).isEmpty()) continue;
-        QComboBox *comboType = twitComboBox(*it, Type);
+        QComboBox *comboType = twitComboBox(*it, Format);
         QLineEdit *editOpts = twitLineEdit(*it, Options);
         if(!comboType || !editOpts) return;
         const QString &type = comboType->currentText();
         const Qt::ItemFlags itflags = (*it)->flags();
         if(comboType->currentData(Qt::UserRole)=="PRESERVE") {
             if((*it)->data(Encrypt, Qt::CheckStateRole).isValid()) {
-                const bool luks = ((*it)->text(Type)=="crypto_LUKS");
+                const bool luks = ((*it)->text(Format)=="crypto_LUKS");
                 (*it)->setCheckState(Encrypt, luks ? Qt::Checked : Qt::Unchecked);
             }
             (*it)->setFlags(itflags & ~Qt::ItemIsUserCheckable);
@@ -386,7 +411,7 @@ void PartMan::treeItemChange(QTreeWidgetItem *item, int column)
             // New items are pre-set with Qt::PartiallyChecked and cannot be excluded.
             QTreeWidgetItem *exclude = (csCrypto==Qt::PartiallyChecked) ? nullptr : item;
             // Set checkboxes and enable the crypto password controls as needed.
-            if(cryptoAny) setEncryptChecks("swap", Qt::Checked, exclude);
+            if(cryptoAny) setEncryptChecks("SWAP", Qt::Checked, exclude);
             if(twitUseFor(item)=="/" && csCrypto==Qt::Checked) {
                 setEncryptChecks("/home", Qt::Checked, exclude);
             }
@@ -400,16 +425,14 @@ void PartMan::treeSelChange()
 {
     QTreeWidgetItem *twit = gui.treePartitions->selectedItems().value(0);
     if(twit) {
-        bool used = true;
         QTreeWidgetItem *drvit = twit->parent();
-        if(!drvit) used = twit->data(Device, Qt::UserRole).toBool();
-        else used = twit->parent()->data(Device, Qt::UserRole).toBool();
+        const bool used = twitIsOldLayout(twit);
         gui.buttonPartClear->setEnabled(!drvit);
-        gui.buttonPartRemove->setEnabled(!used && twit->parent());
+        gui.buttonPartRemove->setEnabled(!used && drvit);
         // Only allow adding partitions if there is enough space.
-        if(used) gui.buttonPartAdd->setEnabled(false);
+        if(!drvit) drvit = twit;
+        if(used && drvit->childCount()>0) gui.buttonPartAdd->setEnabled(false);
         else {
-            if(!drvit) drvit = twit;
             long long maxMB = twitSize(drvit)-PARTMAN_SAFETY_MB;
             for(int ixi = drvit->childCount()-1; ixi >= 0; --ixi) {
                 maxMB -= twitSize(drvit->child(ixi));
@@ -427,31 +450,50 @@ void PartMan::treeMenu(const QPoint &)
 {
     QTreeWidgetItem *twit = gui.treePartitions->selectedItems().value(0);
     if(!twit) return;
+    QMenu menu(gui.treePartitions);
+    QAction *action = nullptr;
+    QAction *actAdd = menu.addAction(tr("&Add partition"));
+    actAdd->setEnabled(gui.buttonPartAdd->isEnabled());
     if(twit->parent()) {
-        if(twitComboBox(twit, Type)->currentText() != "btrfs") return;
-        QMenu menu(gui.treePartitions);
-        const QAction *actBtrfsZlib = menu.addAction(tr("Template: BTRFS compression (ZLIB)"));
-        const QAction *actBtrfsLzo = menu.addAction(tr("Template: BTRFS compression (LZO)"));
-        const QAction *action = menu.exec(QCursor::pos());
-        QLineEdit *editOpts = twitLineEdit(twit, Options);
-        if(action==actBtrfsZlib) editOpts->setText("defaults,noatime,compress-force=zlib");
-        else if(action==actBtrfsLzo) editOpts->setText("defaults,noatime,compress-force=lzo");
-    } else {
-        QMenu menu(gui.treePartitions);
-        menu.addAction(tr("&Reset to on-disk layout"));
+        QComboBox *comboFormat = twitComboBox(twit, Format);
+        const int ixBTRFS = comboFormat->findData("btrfs");
+        QAction *actRemove = menu.addAction(tr("&Remove partition"));
+        actRemove->setEnabled(gui.buttonPartRemove->isEnabled());
         menu.addSeparator();
-        const QAction *actBasic = menu.addAction(tr("Template: &Standard install"));
-        const QAction *actCrypto = menu.addAction(tr("Template: &Encrypted system"));
-        const QAction *action = menu.exec(QCursor::pos());
-        if(action) {
-            if(action==actBasic) layoutDefault(twit, -1, false);
-            else if(action==actCrypto) layoutDefault(twit, -1, true);
-            else { // Reset
-                while(twit->childCount()) twit->removeChild(twit->child(0));
-                populate(twit);
-            }
+        QMenu *menuTemplates = menu.addMenu("&Templates");
+        QAction *actBtrfsZlib = menuTemplates->addAction(tr("BTRFS compression (&ZLIB)"));
+        QAction *actBtrfsLzo = menuTemplates->addAction(tr("BTRFS compression (&LZO)"));
+        if(ixBTRFS<0) {
+            actBtrfsZlib->setEnabled(false);
+            actBtrfsLzo->setEnabled(false);
+        }
+        action = menu.exec(QCursor::pos());
+        if(action==actRemove) partRemoveClick(true);
+        else if(action) {
+            QLineEdit *editOpts = twitLineEdit(twit, Options);
+            comboFormat->setCurrentIndex(ixBTRFS);
+            if(action==actBtrfsZlib) editOpts->setText("defaults,noatime,compress-force=zlib");
+            else if(action==actBtrfsLzo) editOpts->setText("defaults,noatime,compress-force=lzo");
+        }
+    } else {
+        menu.addSeparator();
+        const QAction *actClear = menu.addAction(tr("New &layout"));
+        QAction *actReset = menu.addAction(tr("&Reset layout"));
+        actReset->setDisabled(twitIsOldLayout(twit, false));
+        QMenu *menuTemplates = menu.addMenu("&Templates");
+        const QAction *actBasic = menuTemplates->addAction(tr("&Standard install"));
+        QAction *actCrypto = menuTemplates->addAction(tr("&Encrypted system"));
+        actCrypto->setVisible(gui.gbEncrPass->isVisible());
+        action = menu.exec(QCursor::pos());
+        if(action==actClear) partClearClick(true);
+        else if(action==actBasic) layoutDefault(twit, -1, false);
+        else if(action==actCrypto) layoutDefault(twit, -1, true);
+        else if(action==actReset) {
+            while(twit->childCount()) twit->removeChild(twit->child(0));
+            populate(twit);
         }
     }
+    if(action==actAdd) partAddClick(true);
 }
 
 // Partition manager list buttons
@@ -459,8 +501,8 @@ void PartMan::partClearClick(bool)
 {
     QTreeWidgetItem *twit = gui.treePartitions->selectedItems().value(0);
     if(!twit || twit->parent()) return;
-    while(twit->childCount()) twit->removeChild(twit->child(0));
-    twit->setData(Device, Qt::UserRole, QVariant(false));
+    clearLayout(twit);
+    comboTypeTextChange(QString()); // For the badblocks checkbox.
     treeSelChange();
 }
 void PartMan::partAddClick(bool)
@@ -476,6 +518,7 @@ void PartMan::partAddClick(bool)
     setupItem(part, nullptr);
 
     labelParts(drive);
+    drvitMarkLayout(drive, false);
     part->setSelected(true);
     twit->setSelected(false);
 }
@@ -494,53 +537,45 @@ QWidget *PartMan::composeValidate(bool automatic,
     QString msgConfirm;
     bool encryptAny = false, encryptRoot = false;
     mounts.clear();
-    swaps.clear();
     // Partition use and other validation
-    int mapnum = 0;
+    int mapnum=0, swapnum=0;
     QTreeWidgetItemIterator it(gui.treePartitions, QTreeWidgetItemIterator::NoChildren);
     for(; *it; ++it) {
         QComboBox *comboUse = twitComboBox(*it, UseFor);
         if(!comboUse || comboUse->currentText().isEmpty()) continue;
         QString mount = translateUse(comboUse->currentText());
         const QString &devname = (*it)->text(Device);
-        if(!mount.startsWith("/") && comboUse->findText(mount)<0) {
+        if(!mount.startsWith("/") && comboUse->findText(mount, Qt::MatchFixedString)<0) {
             QMessageBox::critical(master, master->windowTitle(),
                 tr("Invalid use for %1: %2").arg(devname, mount));
             return comboUse;
         }
-        if(mount == "swap") {
-            swaps << *it;
-            mount.clear();
+        if(mount=="SWAP") {
+            ++swapnum;
+            if(swapnum>1) mount+=QString::number(swapnum);
         }
-        // Mount description
-        QString desc;
-        if(mount == "/") desc = "/ (root)";
-        else if(mount.isEmpty()) desc = tr("swap space");
-        else if(mount == "ESP") desc = tr("EFI System Partition");
-        else desc = mount;
         QTreeWidgetItem *twit = mounts.value(mount);
 
         // The mount can only be selected once.
         if(twit) {
-            QMessageBox::critical(master, master->windowTitle(),
-                tr("%1 is already selected for: %2").arg(twit->text(Device), desc));
+            QMessageBox::critical(master, master->windowTitle(), tr("%1 is already"
+                " selected for: %2").arg(twit->text(Device), describeUse(mount)));
             return comboUse;
         } else {
             if(!mount.isEmpty()) mounts.insert(mount, *it);
-            // Warn if using a non-Linux partition (potential install of another OS).
-            const int bdindex = listBlkDevs.findDevice(devname);
-            if (bdindex >= 0 && !listBlkDevs.at(bdindex).isNative) {
-                msgForeignList << devname << desc;
+            if(twitIsOldLayout(*it)) {
+                // Warn if using a non-Linux partition (potential install of another OS).
+                const int bdindex = listBlkDevs.findDevice(devname);
+                if (bdindex >= 0 && !listBlkDevs.at(bdindex).isNative) {
+                    msgForeignList << devname << describeUse(mount);
+                }
             }
         }
         QVariant mapperData;
         if((*it)->checkState(Encrypt) == Qt::Checked) {
             if(mount == "/") mapperData = "root.fsm";
-            else if(mount.isEmpty()) {
-                const int swapnum = swaps.count();
-                if(swapnum <= 1) mapperData = "swap.fsm";
-                else mapperData = QString("swap%1.fsm").arg(swapnum);
-            } else {
+            else if(mount.isEmpty()) mapperData = mount.toLower();
+            else {
                 mapperData = QString::number(++mapnum)
                     + mount.replace('/','.') + ".fsm";
             }
@@ -581,22 +616,16 @@ QWidget *PartMan::composeValidate(bool automatic,
     if(!automatic) {
         QString msg;
         QStringList msgFormatList;
-        // Format (vs just configuring) swap partitions?
-        for(QTreeWidgetItem *swap : swaps) {
-            const QString &dev = swap->text(Device);
-            if (twitWillFormat(swap)) msgFormatList << dev << tr("swap space");
-            else msgConfirm += " - " + tr("Configure %1 as swap space").arg(dev) + "\n";
-        }
-        // Format (vs just mounting) other partitions?
+        // Format (vs just mounting or configuring) partitions.
         for(const auto &it : mounts.toStdMap()) {
             const QString &dev = it.second->text(Device);
-            if(twitWillFormat(it.second)) msgFormatList << dev << it.first;
+            if(twitWillFormat(it.second)) msgFormatList << dev << describeUse(it.first);
             else if(it.first == "/") {
                 msgConfirm += " - " + tr("Delete the data on %1"
                     " except for /home").arg(dev) + "\n";
             } else {
-                msgConfirm += " - " + tr("Reuse (no reformat) %1 as the"
-                    " %2 partition").arg(dev, it.first) + "\n";
+                msgConfirm += " - " + tr("Reuse (no reformat) %1 as"
+                    " %2").arg(dev, describeUse(it.first)) + "\n";
             }
         }
 
@@ -607,7 +636,7 @@ QWidget *PartMan::composeValidate(bool automatic,
             msg += tr("The following partitions you selected are not Linux partitions:") + "\n\n";
             for (QStringList::Iterator it = msgForeignList.begin(); it != msgForeignList.end(); ++it) {
                 QString &s = *it;
-                msg += msgPartSel.arg(s).arg((QString)*(++it));
+                msg += msgPartSel.arg(s, describeUse(*(++it)));
             }
             msg += "\n";
         }
@@ -623,7 +652,7 @@ QWidget *PartMan::composeValidate(bool automatic,
             msg += tr("The %1 installer will now format and destroy the data on the following partitions:").arg(project) + "\n\n";
             for (QStringList::Iterator it = msgFormatList.begin(); it != msgFormatList.end(); ++it) {
                 QString &s = *it;
-                msg += msgPartSel.arg(s).arg((QString)*(++it));
+                msg += msgPartSel.arg(s, describeUse(*(++it)));
             }
             if (!msgConfirm.isEmpty()) msg += "\n";
         }
@@ -649,7 +678,7 @@ bool PartMan::calculatePartBD()
     const int driveCount = gui.treePartitions->topLevelItemCount();
     for(int ixDrive = 0; ixDrive < driveCount; ++ixDrive) {
         QTreeWidgetItem *drvit = gui.treePartitions->topLevelItem(ixDrive);
-        const bool useExist = drvit->data(Device, Qt::UserRole).toBool();
+        const bool useExist = twitIsOldLayout(drvit, false);
         QString drv = drvit->text(Device);
         int ixDriveBD = listBlkDevs.findDevice(drv);
         const int partCount = drvit->childCount();
@@ -674,7 +703,7 @@ bool PartMan::calculatePartBD()
             if(useFor.isEmpty()) continue;
             QString bdFS;
             if(twit->checkState(Encrypt) == Qt::Checked) bdFS = QStringLiteral("crypto_LUKS");
-            else bdFS = twitComboBox(twit, Type)->currentText();
+            else bdFS = twitComboBox(twit, Format)->currentText();
             if(useExist) {
                 BlockDeviceInfo &bdinfo = listBlkDevs[ixPartBD];
                 listToUnmount << bdinfo.name;
@@ -801,17 +830,8 @@ int PartMan::layoutDefault(QTreeWidgetItem *drivetree,
     int rootPercent, bool crypto, bool updateTree)
 {
     if(rootPercent<0) rootPercent = gui.sliderPart->value();
-    if(updateTree) {
-        // Clear the existing layout from the target device.
-        while(drivetree->childCount()) drivetree->removeChild(drivetree->child(0));
-        // Mark the drive as "unused" as its layout will be replaced.
-        drivetree->setData(Device, Qt::UserRole, QVariant(false));
-    }
+    if(updateTree) clearLayout(drivetree);
     // Drive geometry basics.
-    bool ok = true;
-    const QString &fsText = gui.freeSpaceEdit->text().trimmed();
-    int free = fsText.isEmpty() ? 0 : fsText.toInt(&ok,10);
-    if (!ok) return 0;
     const long long driveSize = twitSize(drivetree);
     int rootFormatSize = driveSize-PARTMAN_SAFETY_MB;
     // Boot partitions.
@@ -842,10 +862,6 @@ int PartMan::layoutDefault(QTreeWidgetItem *drivetree,
     if(swapMaxMB > 8192) swapMaxMB = 8192; // 8GB cap for the whole calculation.
     if(swapFormatSize > swapMaxMB) swapFormatSize = swapMaxMB;
     rootFormatSize -= swapFormatSize;
-    if (free > 0 && rootFormatSize > 8192) {
-        if (free > (rootFormatSize - 8192)) free = rootFormatSize - 8192;
-        rootFormatSize -= free;
-    }
     // Home
     int homeFormatSize = rootFormatSize;
     rootFormatSize = (rootFormatSize * rootPercent) / 100;
@@ -867,7 +883,7 @@ int PartMan::countPrepSteps()
     // Creation of new partitions
     for(int ixi = gui.treePartitions->topLevelItemCount(); ixi>=0; --ixi) {
         QTreeWidgetItem *drvit = gui.treePartitions->topLevelItem(ixi);
-        if(twitComboBox(drvit, Type)) pcount += drvit->childCount() + 1; // Partition table creation included.
+        if(twitComboBox(drvit, Format)) pcount += drvit->childCount() + 1; // Partition table creation included.
     }
     // Formatting partitions
     pcount += mounts.count();
@@ -877,11 +893,6 @@ int PartMan::countPrepSteps()
         QTreeWidgetItem *twit = mi.value();
         if(twit->checkState(Encrypt)==Qt::Checked) ++pcount; //LUKS format.
     }
-    // Formatting swap space
-    pcount += swaps.count();
-    for(QTreeWidgetItem *twit : swaps) {
-        if(twit->checkState(Encrypt)==Qt::Checked) ++pcount; //LUKS format.
-    }
     // Final count
     return pcount;
 }
@@ -924,19 +935,14 @@ bool PartMan::preparePartitions()
     qDebug() << " ---- PARTITION FORMAT SCHEDULE ----";
     for(const auto &it : mounts.toStdMap()) {
         QTreeWidgetItem *twit = it.second;
-        qDebug().nospace().noquote() << twit->text(UseFor) << ": "
-            << twit->text(Device) << " (" << twitSize(twit) << "MB)";
-    }
-    for(int ixi = swaps.count()-1; ixi>=0; --ixi) {
-        QTreeWidgetItem *twit = swaps.at(ixi);
-        qDebug().nospace().noquote() << "swap" << ixi << ": "
+        qDebug().nospace().noquote() << it.first << ": "
             << twit->text(Device) << " (" << twitSize(twit) << "MB)";
     }
 
     // Clear the existing partition tables on devices which will have a new layout.
     for(int ixi = gui.treePartitions->topLevelItemCount()-1; ixi>=0; --ixi) {
         QTreeWidgetItem *twit = gui.treePartitions->topLevelItem(ixi);
-        if(twit->data(Device, Qt::UserRole).toBool()) continue;
+        if(twitIsOldLayout(twit, false)) continue;
         const QString &drv = twit->text(Device);
         proc.status(tr("Clearing existing partition tables"));
         clearPartitionTables(drv);
@@ -951,7 +957,7 @@ bool PartMan::preparePartitions()
         QTreeWidgetItem *drvitem = gui.treePartitions->topLevelItem(ixi);
         const QString &drvdev = drvitem->text(Device);
         const int devCount = drvitem->childCount();
-        if(drvitem->data(Device, Qt::UserRole).toBool()) {
+        if(twitIsOldLayout(drvitem, true)) {
             // Using existing partitions.
             QString cmd; // command to set the partition type
             const int ixBlkDev = listBlkDevs.findDevice(drvdev);
@@ -964,7 +970,7 @@ bool PartMan::preparePartitions()
             for(int ixdev=0; ixdev<devCount; ++ixdev) {
                 QTreeWidgetItem *twit = drvitem->child(ixdev);
                 if(twitUseFor(twit).isEmpty()) continue;
-                const QStringList devsplit(BlockDeviceInfo::split("/dev/" + twit->text(Device)));
+                const QStringList &devsplit = BlockDeviceInfo::split("/dev/" + twit->text(Device));
                 if(!proc.exec(cmd.arg(devsplit.at(0), devsplit.at(1)))) return false;
                 proc.sleep(1000);
             }
@@ -1001,18 +1007,6 @@ bool PartMan::formatPartitions()
     const QByteArray &encPass = (gui.entireDiskButton->isChecked()
                                  ? gui.FDEpassword : gui.FDEpassCust)->text().toUtf8();
     const QString &statup = tr("Setting up LUKS encrypted containers");
-    // Swap partitions.
-    for(QTreeWidgetItem *twit : swaps) {
-        const QString dev = "/dev/" + twit->text(Device);
-        if(twit->checkState(Encrypt)!=Qt::Checked) continue;
-        if(twitWillFormat(twit)) {
-            proc.status(statup);
-            if(!luksMake(dev, encPass)) return false;
-        }
-        proc.status(statup);
-        if (!luksOpen(dev, twitMappedDevice(twit), encPass)) return false;
-    }
-    // Other partitions.
     for(QTreeWidgetItem *twit : mounts) {
         const QString dev = "/dev/" + twit->text(Device);
         if(twit->checkState(Encrypt)!=Qt::Checked) continue;
@@ -1024,31 +1018,27 @@ bool PartMan::formatPartitions()
         if (!luksOpen(dev, twitMappedDevice(twit), encPass)) return false;
     }
 
-    // Format all swaps.
-    for(int ixi = swaps.count()-1; ixi>=0; --ixi) {
-        QTreeWidgetItem *twit = swaps.at(ixi);
-        if(!twitWillFormat(twit)) continue;
-        proc.status(tr("Formatting swap partitions"));
-        QString cmd("/sbin/mkswap " + twitMappedDevice(twit, true));
-        const QString &label = twitLineEdit(twit, Label)->text();
-        if(!label.isEmpty()) cmd.append(" -L \"" + label + "\"");
-        if(!proc.exec(cmd, true)) return false;
-    }
-    // Format other partitions.
+    // Format partitions.
     const bool badblocks = gui.badblocksCheck->isChecked();
     for(const auto &it : mounts.toStdMap()) {
         QTreeWidgetItem *twit = it.second;
         if(!twitWillFormat(twit)) continue;
-        proc.status(tr("Formatting: %1").arg(it.first));
+        proc.status(tr("Formatting: %1").arg(describeUse(it.first)));
         const QString &dev = twitMappedDevice(twit, true);
         const QString &useFor = translateUse(twitComboBox(twit, UseFor)->currentText());
         if(useFor=="ESP") {
-            proc.status(tr("Formatting EFI System Partition"));
             if (!proc.exec("mkfs.msdos -F 32 " + dev)) return false;
             // Sets boot flag and ESP flag.
-            proc.exec("parted -s /dev/" + BlockDeviceInfo::split(dev).at(0) + " set 1 esp on");
+            const QStringList &devsplit = BlockDeviceInfo::split(dev);
+            proc.exec("parted -s /dev/" + devsplit.at(0)
+                + " set " + devsplit.at(1) + " esp on");
+        } else if(useFor.startsWith("SWAP")) {
+            QString cmd("/sbin/mkswap " + twitMappedDevice(twit, true));
+            const QString &label = twitLineEdit(twit, Label)->text();
+            if(!label.isEmpty()) cmd.append(" -L \"" + label + "\"");
+            if(!proc.exec(cmd, true)) return false;
         } else {
-            if(!formatLinuxPartition(dev, twitComboBox(twit, Type)->currentText(),
+            if(!formatLinuxPartition(dev, twitComboBox(twit, Format)->currentText(),
                 badblocks, twitLineEdit(twit, Label)->text())) return false;
         }
         proc.sleep(1000);
@@ -1129,23 +1119,15 @@ bool PartMan::fixCryptoSetup(const QString &keyfile, bool isNewKey)
         const QString &dev = it.second->text(Device);
         QString uuid = proc.execOut(cmdBlkID + dev);
         out << twitMappedDevice(it.second, false) << " /dev/disk/by-uuid/" << uuid;
-        if(noKey || it.first==keyMount) out << " none luks \n";
+        if(noKey || it.first==keyMount) out << " none";
         else {
             if(isNewKey) {
                 if(!proc.exec(cmdAddKey.arg(dev), true, &password)) return false;
             }
-            out << ' ' << keyfile << " luks \n";
+            out << ' ' << keyfile;
         }
-    }
-    // Swap spaces
-    for(QTreeWidgetItem *twit : swaps) {
-        if(twit->checkState(Encrypt)!=Qt::Checked) continue;
-        const QString &dev = twit->text(Device);
-        QString uuid = proc.execOut(cmdBlkID + dev);
-        if(!proc.exec(cmdAddKey.arg(dev), true, &password)) return false;
-        out << twitMappedDevice(twit, false) << " /dev/disk/by-uuid/" << uuid;
-        if(noKey) out << " none luks,nofail \n";
-        else out << ' ' << keyfile << " luks,nofail \n";
+        if(!it.first.startsWith("SWAP")) out << " luks \n";
+        else out << " luks,nofail \n";
     }
     // Update cryptdisks if the key is not in the root file system.
     if(keyMount!='/') {
@@ -1164,9 +1146,9 @@ bool PartMan::makeFstab(bool populateMediaMounts)
     QTextStream out(&file);
     out << "# Pluggable devices are handled by uDev, they are not in fstab\n";
     const QString cmdBlkID("blkid -o value UUID -s UUID ");
-    // File systems
+    // File systems and swap space.
     for(auto &it : mounts.toStdMap()) {
-        if(it.first == "EFI") continue; // EFI will be dealt with later.
+        if(it.first == "ESP") continue; // EFI will be dealt with later.
         const QString &dev = twitMappedDevice(it.second, true);
         qDebug() << "Creating fstab entry for:" << it.first << dev;
         // Device ID or UUID
@@ -1174,24 +1156,19 @@ bool PartMan::makeFstab(bool populateMediaMounts)
         else out << "UUID=" + proc.execOut(cmdBlkID + dev);
         // Mount point, file system
         QString fstype = proc.execOut("blkid " + dev + " -o value -s TYPE");
-        out << " " << it.first << " " << fstype;
+        if(fstype=="swap") out << " swap swap";
+        else out << ' ' << it.first << ' ' << fstype;
         // Options
-        out << " " << twitLineEdit(it.second, Options)->text();
+        out << ' ' << twitLineEdit(it.second, Options)->text();
         // Dump, pass
-        if(fstype.startsWith("reiser")) out << " 0 0";
+        if(fstype=="swap") out << " 0 0\n";
+        else if(fstype.startsWith("reiser")) out << " 0 0\n";
         else if(it.first=="/boot") out << " 1 1\n";
         else if(it.first=="/") {
             if(fstype.startsWith("btrfs")) out << " 1 0\n";
             else out << " 1 1\n";
         } else out << " 1 2\n";
     }
-    // Swap spaces
-    for(QTreeWidgetItem *twit : swaps) {
-        const QString &dev = twitMappedDevice(twit, true);
-        if(twitIsMapped(twit)) out << dev;
-        else out << "UUID=" + proc.execOut(cmdBlkID + dev);
-        out << " swap swap " << twitLineEdit(twit, Options)->text() << " 0 0\n";
-    }
     // EFI System Partition
     if (gui.grubEspButton->isChecked()) {
         const QString espdev = "/dev/" + gui.grubBootCombo->currentData().toString();
@@ -1215,7 +1192,7 @@ bool PartMan::mountPartitions()
         const QString point("/mnt/antiX" + it.first);
         const QString &dev = twitMappedDevice(it.second, true);
         proc.status(tr("Mounting: %1").arg(dev));
-        mkdir(point.toUtf8(), 0755);
+        if(!proc.mkpath(point)) return false;
         if(it.first=="/boot") {
              // needed to run fsck because sfdisk --part-type can mess up the partition
             if(!proc.exec("fsck.ext4 -y " + dev)) return false;
@@ -1229,16 +1206,14 @@ bool PartMan::mountPartitions()
 
 void PartMan::unmount(bool all)
 {
-    for(QTreeWidgetItem *twit : swaps) {
-        if(!(all || twit->checkState(Encrypt)==Qt::Checked)) continue;
-        QString cmd("cryptsetup close %1");
-        proc.exec(cmd.arg(twitMappedDevice(twit)), true);
-    }
     QMapIterator<QString, QTreeWidgetItem *> it(mounts);
     it.toBack();
     while(it.hasPrevious()) {
         it.previous();
         if(it.key().at(0)!='/') continue;
+        if(!it.key().startsWith("SWAP")) {
+            proc.exec("/bin/umount -l /mnt/antiX" + it.key(), true);
+        }
         proc.exec("/bin/umount -l /mnt/antiX" + it.key(), true);
         QTreeWidgetItem *twit = it.value();
         if(!(all || twit->checkState(Encrypt)==Qt::Checked)) continue;
@@ -1263,27 +1238,46 @@ QString PartMan::getMountDev(const QString &point, const bool mapped)
 }
 int PartMan::swapCount()
 {
-    return swaps.count();
+    int count = 0;
+    for(const auto &mount : mounts.toStdMap()) {
+        if(mount.first.startsWith("SWAP")) ++count;
+    }
+    return count;
 }
 int PartMan::isEncrypt(const QString &point)
 {
     int count = 0;
-    if(point.isEmpty() || point=="SWAP") {
-        for(QTreeWidgetItem *twit : swaps) {
-            if(twit->checkState(Encrypt)==Qt::Checked) ++count;
-        }
-    }
     if(point.isEmpty()) {
         for(QTreeWidgetItem *twit : mounts) {
             if(twit->checkState(Encrypt)==Qt::Checked) ++count;
         }
+    } else if(point=="SWAP") {
+        for(const auto &mount : mounts.toStdMap()) {
+            if(mount.first.startsWith("SWAP")
+                && mount.second->checkState(Encrypt)==Qt::Checked) ++count;
+        }
+    } else {
+        const QTreeWidgetItem *twit = mounts.value(point);
+        if(twit && twit->checkState(Encrypt)==Qt::Checked) ++count;
     }
-    const QTreeWidgetItem *twit = mounts.value(point);
-    if(twit && twit->checkState(Encrypt)==Qt::Checked) ++count;
     return count;
 }
 
 // Helpers
+inline void PartMan::drvitMarkLayout(QTreeWidgetItem *drvit, const bool old)
+{
+    if(old) drvit->setIcon(Device, QIcon());
+    else drvit->setIcon(Device, QIcon(":/appointment-soon"));
+    gui.treePartitions->resizeColumnToContents(Device);
+}
+inline bool PartMan::twitIsOldLayout(const QTreeWidgetItem *twit, const bool chkUp) const
+{
+    if(chkUp) {
+        const QTreeWidgetItem *drvit = twit->parent();
+        if(drvit) return drvit->icon(Device).isNull();
+    }
+    return twit->icon(Device).isNull();
+}
 inline long long PartMan::twitSize(QTreeWidgetItem *twit, bool bytes)
 {
     QWidget *spin = gui.treePartitions->itemWidget(twit, Size);
@@ -1299,7 +1293,7 @@ inline long long PartMan::twitSize(QTreeWidgetItem *twit, bool bytes)
 }
 inline bool PartMan::twitWillFormat(QTreeWidgetItem *twit)
 {
-    QComboBox *combo = twitComboBox(twit, Type);
+    QComboBox *combo = twitComboBox(twit, Format);
     return (combo->currentData(Qt::UserRole) != "PRESERVE");
 }
 inline QString PartMan::twitUseFor(QTreeWidgetItem *twit)
